import React, { useEffect, useState } from 'react'
import { Icons } from '../Icons'
import { LiveProvider, LiveError, LivePreview } from 'react-live'
import CodeEditor from '../CodeEditor'
import useKeyPress from '@/hooks/useKeyPress'
import { toast } from 'react-toastify'
import presets from '@/utils/globalPresets'

const CodePlayground = ({ code, onChange, scope }) => {
  const [view, setView] = useState('preview')
  const [currentCode, setCurrentCode] = useState(code)

  const toggleView = () => {
    setView(view === 'preview' ? 'code' : 'preview')
  }

  useKeyPress('Tab', toggleView, true)

  const copyToClipboard = () => {
    navigator.clipboard.writeText(code).then(() => {
      toast.success('Código copiado al portapapeles', presets.toaster)
    }).catch(err => {
      console.error('Algo salió mal al copiar el código: ', err)
    })
  }

  const handleCodeChange = (editor, data, value) => {
    setCurrentCode(editor)
    onChange(editor)
  }

  useEffect(() => {
    setCurrentCode(code)
  }, [code])

  return (
    <div className='flex-row'>
      <div className='mb-3 flex justify-end'>
        <div className="shadow-neon shadow-neon-sky flex space-x-1 rounded-lg bg-slate-200 dark:!bg-slate-900">
          <button
            onClick={() => setView('preview')}
            className={`flex items-center rounded-md px-2 py-[0.4375rem] pr-3 text-sm font-semibold ${view === 'preview' ? 'bg-white shadow dark:!bg-[#333333]' : 'bg-slate-200 dark:!bg-transparent'}`}
          >
            <Icons.Eye className={`h-5 w-5 flex-none ${view === 'preview' ? 'stroke-sky-500 dark:stroke-sky-300' : 'stroke-slate-600 dark:stroke-slate-400'}`}/>
            <span className={`ml-2 ${view === 'preview' ? 'text-sky-500 dark:text-sky-300' : 'text-slate-600 dark:text-slate-400'}`}>
                Preview
            </span>
          </button>
          <button
            onClick={() => setView('code')}
            className={`flex items-center rounded-md px-2 py-[0.4375rem] pr-3 text-sm font-semibold ${view === 'code' ? 'bg-white shadow dark:!bg-[#333333]' : 'bg-slate-200 dark:!bg-transparent'}`}
          >
            <Icons.Code className={`h-5 w-5 flex-none ${view === 'code' ? 'stroke-sky-500 dark:stroke-sky-300' : 'stroke-slate-600 dark:stroke-slate-400'}`}/>
            <span className={`ml-2 ${view === 'code' ? 'text-sky-500 dark:text-sky-300' : 'text-slate-600 dark:text-slate-400'}`}>
                Code
            </span>
          </button>
        </div>

        <div className='my-1 ml-6 mr-3 h-auto w-[2px] bg-slate-900/10 dark:bg-slate-900/20' />

        <button
          onClick={copyToClipboard}
          className="shadow-neon shadow-neon-sky group relative flex h-9 w-9 items-center justify-center rounded-md bg-white text-slate-600 hover:text-slate-900 dark:!bg-[#333333] dark:!stroke-sky-300 dark:hover:!text-sky-100"
        >
          <Icons.Copy className="h-5 w-5 stroke-slate-400 transition group-hover:rotate-[-4deg] group-hover:stroke-slate-600 group-active:group-hover:stroke-sky-500 dark:group-hover:stroke-sky-300" />
        </button>
      </div>

      <div>
        <LiveProvider code={currentCode} scope={scope} noInline={true}>
          { view === 'preview' &&
              <div className={`${view === 'preview' ? '' : 'hidden'}`}>
                <LivePreview className='shadow-neon rounded-lg border p-2' />
                <LiveError className="mt-2 rounded bg-red-500 text-white" />
              </div>
          }

          <div className={`${view === 'code' ? '' : 'hidden'}`}>
            <CodeEditor code={currentCode} onChange={handleCodeChange} />
          </div>
        </LiveProvider>
      </div>
    </div>
  )
}

export default CodePlayground
