import { useCallback, useEffect, useState } from 'react'

interface ScreenType {
  isLargeDesktop: boolean;
  isDesktop: boolean;
  isMobile: boolean;
  isTablet: boolean;
}

/**
 * Hook personalizado de React para determinar el tipo de pantalla basado en el ancho del viewport.
 *
 * Este hook es útil para el diseño responsivo, permitiendo ajustar la interfaz de usuario
 * y la lógica del componente según el tamaño de la pantalla. Utiliza la API de `window` para
 * escuchar cambios en el tamaño de la ventana y ajusta el estado del tipo de pantalla en consecuencia.
 *
 * Los breakpoints para mobile, tablet, desktop y large desktop son configurables según las necesidades
 * del proyecto.
 *
 * @returns {ScreenType} Objeto que contiene booleanos para cada tipo de pantalla (mobile, tablet, desktop, largeDesktop).
 *
 * @example
 * import React from 'react';
 * import useScreenType from './useScreenType';
 *
 * const MyComponent: React.FC = () => {
 *   const screenType = useScreenType();
 *
 *   return (
 *     <div>
 *       {screenType.isMobile ? <MobileComponent /> : <DesktopComponent />}
 *     </div>
 *   );
 * };
 *
 * export default MyComponent;
 */
export const useScreenType = (): ScreenType => {
  // Estado inicial para SSR o cliente
  const initialState: ScreenType = {
    isLargeDesktop: false,
    isDesktop: false,
    isMobile: true,
    isTablet: false
  }

  const [screenType, setScreenType] = useState<ScreenType>(initialState)

  const getScreenType = useCallback((): ScreenType => {
    if (typeof window !== 'undefined') {
      const width = window.innerWidth
      return {
        isLargeDesktop: width >= 1200,
        isDesktop: width >= 992 && width < 1200,
        isTablet: width >= 768 && width < 992,
        isMobile: width < 768
      }
    }
    return initialState
  }, [])

  useEffect(() => {
    if (typeof window !== 'undefined') {
      const handleResize = () => {
        setScreenType(getScreenType())
      }

      handleResize()

      window.addEventListener('resize', handleResize)

      return () => window.removeEventListener('resize', handleResize)
    }
  }, [getScreenType])

  return screenType
}
