import React from 'react'

/**
 * Hook de React para obtener el ancho y alto de la ventana del navegador.
 *
 * @example
 *
 * import React from 'react'
 *
 * const MyComponent: React.FC = () => {
 *  const { width, height } = useWindowSize()
 *  return (
 *   <div>
 *    <p>Ancho: {width}</p>
 *    <p>Alto: {height}</p>
 *   </div>
 *  )}
 *
 * export default MyComponent
 *
 * @returns {object} Retorna un objeto con las propiedades `width` y `height` que representan el ancho y alto de la ventana del navegador.
 */
export function useWindowSize (): { width: number, height: number } {
  const [size, setSize] = React.useState({
    width: null,
    height: null
  })

  React.useLayoutEffect(() => {
    const handleResize = () => {
      setSize({
        width: window.innerWidth,
        height: window.innerHeight
      })
    }

    handleResize()
    window.addEventListener('resize', handleResize)

    return () => {
      window.removeEventListener('resize', handleResize)
    }
  }, [])

  return size
}
